﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class Transacoes : MonoBehaviour {
	//public static int random;
	public GUIStyle font;
	//Define o numero maximo de simbolos de um pedido
	public int tamMaximoPedido = 2;
	//lista dos simbolos atuais
	public Queue pedido;
	//fila de tipos do simbolo atual
	public Queue<TipoSimbolos> filaTiposPedido;
	//Pontos
	public int pontos;
	public int objetivoDePontos = 10;

	//Pontuacao por acerto
	int pontosPorAcerto = 1;

	//Define evento de fim de jogo
	public delegate void OnGameOver();
	public event OnGameOver GameOver;

	//Define evento de troca de tipo de simbolo
	public delegate void OnTrocaTipoSimbolo(TipoSimbolos tipo);
	public event OnTrocaTipoSimbolo TrocaTipo;

	//Define listas de tipos
	private List<Simbolos> produtos;
	private List<Simbolos> quantidade;
	private List<Simbolos> tempo;
	private Dictionary<TipoSimbolos, List<Simbolos>> dicionarioTipoSimbolos; 


	//Garante que Transacoes tenha uma unica instancia
	//Implementacao de singleton
	private static Transacoes instance;
	public static Transacoes Instance
	{
		get
		{
			if(instance == null)
				instance = GameObject.FindObjectOfType(typeof(Transacoes)) as Transacoes;	
			return instance;
		}
	}

	void Start () {
		filaTiposPedido = new Queue<TipoSimbolos>();
		pedido = new Queue();
		MudaPedido ();
		font.normal.textColor = Color.blue;
		font.fontSize = 14;

		//Inicializa listas de tipos
		produtos = new List<Simbolos>();
		quantidade = new List<Simbolos>();
		tempo = new List<Simbolos>();
		dicionarioTipoSimbolos = new Dictionary<TipoSimbolos, List<Simbolos>>();

		//Adiciona Simbolos a suas listas
		produtos.Add(Simbolos.Burro);
		produtos.Add(Simbolos.Carneiro);
		produtos.Add (Simbolos.Cevada);
		produtos.Add(Simbolos.Pao);
		produtos.Add (Simbolos.Peixe);
		produtos.Add (Simbolos.Vaca);

		quantidade.Add (Simbolos.Um);
		quantidade.Add (Simbolos.Cinco);
		quantidade.Add (Simbolos.Trinta);
		quantidade.Add (Simbolos.Trezentos);
		quantidade.Add (Simbolos.Novecentos);
		quantidade.Add (Simbolos.NoveMil);

		tempo.Add (Simbolos.UmMes);
		tempo.Add (Simbolos.DoisMeses);
		tempo.Add (Simbolos.TresMeses);
		tempo.Add (Simbolos.UmAno);
		tempo.Add (Simbolos.DoisAnos);
		tempo.Add (Simbolos.OnzeMeses);

		//Adiciona listas no dicionario
		dicionarioTipoSimbolos.Add(TipoSimbolos.Produto, produtos);
		dicionarioTipoSimbolos.Add (TipoSimbolos.Quantidade, quantidade);
		dicionarioTipoSimbolos.Add (TipoSimbolos.Tempo,tempo);
	}

	void OnGUI(){
		GUILayout.BeginArea (new Rect (0, 0, 700, 700));
		GUILayout.Label ("Pontuaçao: " + this.pontos, font);
		foreach (string simbolo in pedido)
		{
			GUILayout.Label (simbolo, font); 
		}
		GUILayout.EndArea ();
	}

	//Adiciona pontuaçao por acerto
	void pontua()
	{
		this.pontos += pontosPorAcerto;
	}

	public void MudaPedido(){
		pedido.Clear ();
		int tamPedido = Random.Range (1, this.tamMaximoPedido+1);

		for (int i = 0; i < tamPedido; i++) 
		{
			//Cria um valor randomico de 0 ate o numero de valores do Enum de tipos
			int random = Random.Range (0, Simbolos.GetValues(typeof(Simbolos)).Length);
			Simbolos tipo = (Simbolos) random;
			pedido.Enqueue(tipo.ToString());
		}
		atualizaFilaPedidos();
	}
	public void atualizaFilaPedidos(){
		filaTiposPedido.Clear();
		foreach(Simbolos e in pedido)
		{
			TipoSimbolos tipo;
			if(quantidade.Contains(e))
			{
				tipo = TipoSimbolos.Quantidade;
			}
			else if(produtos.Contains(e))
			{
				tipo = TipoSimbolos.Produto;
			}
			else
			{
				tipo = TipoSimbolos.Tempo;
			}
			filaTiposPedido.Enqueue(tipo);
		}
		TrocaTipo(filaTiposPedido.Dequeue());
	}

	public void ProximoSimbolo(){
		TipoSimbolos tipo = filaTiposPedido.Dequeue();
		if(tipo != null)
			TrocaTipo(tipo);
	}
	
	public bool ConfirmaEscrita(Queue resposta)
	{
		bool certo = false;
		
		//Verifica se o tamanho da resposta e diferente do tamanho do pedido
		if (resposta.Count != pedido.Count)
		{
			certo = false;
		}
		else {

			//Compara pedido e resposta
			while(pedido.Count > 0)
			{
				string ped = (string) pedido.Dequeue();
				string resp = (string) resposta.Dequeue();
				if(ped.Equals(resp))
					certo = true;
				else{
					certo = false;
					break;
				}
			}
		}
		
		if (certo) {
			pontua();
		}

		TabletEscrita.Instance.Destruir();
		MudaPedido();

		return certo;
	}

	public void FinalizaJogo()
	{
		GameOver();
	}
}
